@@ -49,7 +49,8 @@ typedef struct
     void *hash;
     regidx_t *reg_idx;
     regitr_t *reg_itr;
-    int argc, region_is_file, target_is_file, nsites, chunk_cnt, rec_cnt, scatter_is_file, output_type, n_threads;
+    int argc, region_is_file, target_is_file, nsites, chunk_cnt, rec_cnt, scatter_is_file, output_type, n_threads, clevel;
+    int regions_overlap, targets_overlap;
     char **argv, *region, *target, *scatter, *fname, *prefix, *extra, *output_dir;
     bcf_srs_t *sr;
     kstring_t str;
@@ -58,6 +59,7 @@ typedef struct
     int record_cmd_line;
     char **hts_opts;
     int nhts_opts;
+    bcf_hdr_t *hdr;
 }
 args_t;
 
@@ -74,23 +76,25 @@ static const char *usage_text(void)
         "\n"
         "Usage: bcftools +scatter [Options]\n"
         "Plugin options:\n"
-        "   -e, --exclude <expr>          exclude sites for which the expression is true\n"
-        "   -i, --include <expr>          select sites for which the expression is true\n"
-        "       --no-version              do not append version and command line to the header\n"
-        "   -o, --output DIR              write output to the directory DIR\n"
-        "   -O, --output-type b|u|z|v     b: compressed BCF, u: uncompressed BCF, z: compressed VCF, v: uncompressed VCF [v]\n"
-        "       --threads <int>           use multithreading with <int> worker threads [0]\n"
-        "   -r, --regions <region>        restrict to comma-separated list of regions\n"
-        "   -R, --regions-file <file>     restrict to regions listed in a file\n"
-        "   -t, --targets [^]<region>     similar to -r but streams rather than index-jumps. Exclude regions with \"^\" prefix\n"
-        "   -T, --targets-file [^]<file>  similar to -R but streams rather than index-jumps. Exclude regions with \"^\" prefix\n"
-        "   -n, --nsites-per-chunk <int>  keep N sites for each chunk\n"
-        "   -s, --scatter <region>        comma-separated list of regions defining variant windows for each output VCF\n"
-        "   -S, --scatter-file <file>     regions listed in a file with an optional second column used to name each output VCF.\n"
-        "                                     Variants from multiple regions can be assigned to the same output VCF.\n"
-        "   -x, --extra <string>          output records not overlapping listed regions in separate file\n"
-        "   -p, --prefix <string>         prepend string to output VCF names\n"
-        "       --hts-opts <list>         low-level options to pass to HTSlib, e.g. block_size=32768\n"
+        "   -e, --exclude EXPR              Exclude sites for which the expression is true\n"
+        "   -i, --include EXPR              Select sites for which the expression is true\n"
+        "       --no-version                Do not append version and command line to the header\n"
+        "   -o, --output DIR                Write output to the directory DIR\n"
+        "   -O, --output-type u|b|v|z[0-9]  u/b: un/compressed BCF, v/z: un/compressed VCF, 0-9: compression level [v]\n"
+        "       --threads INT               Use multithreading with INT worker threads [0]\n"
+        "   -r, --regions REGION            Restrict to comma-separated list of regions\n"
+        "   -R, --regions-file FILE         Restrict to regions listed in a file\n"
+        "       --regions-overlap 0|1|2     Include if POS in the region (0), record overlaps (1), variant overlaps (2) [1]\n"
+        "   -t, --targets [^]REGION         Similar to -r but streams rather than index-jumps. Exclude regions with \"^\" prefix\n"
+        "   -T, --targets-file [^]FILE      Similar to -R but streams rather than index-jumps. Exclude regions with \"^\" prefix\n"
+        "       --targets-overlap 0|1|2     Include if POS in the region (0), record overlaps (1), variant overlaps (2) [0]\n"
+        "   -n, --nsites-per-chunk INT      Keep N sites for each chunk\n"
+        "   -s, --scatter REGION            Comma-separated list of regions defining variant windows for each output VCF\n"
+        "   -S, --scatter-file FILE         Regions listed in a file with an optional second column used to name each output VCF.\n"
+        "                                        Variants from multiple regions can be assigned to the same output VCF.\n"
+        "   -x, --extra STRING              Output records not overlapping listed regions in separate file\n"
+        "   -p, --prefix STRING             Prepend string to output VCF names\n"
+        "       --hts-opts LIST             Low-level options to pass to HTSlib, e.g. block_size=32768\n"
         "\n"
         "Examples:\n"
         "   # Scatter a VCF file by shards with 10000 variants each\n"
@@ -175,7 +179,10 @@ static void open_set(subset_t *set, args_t *args)
     if ( args->output_type & FT_BCF ) kputs(".bcf", &args->str);
     else if ( args->output_type & FT_GZ ) kputs(".vcf.gz", &args->str);
     else kputs(".vcf", &args->str);
-    set->fh = hts_open(args->str.s, hts_bcf_wmode2(args->output_type,args->str.s));
+
+    char wmode[8];
+    set_wmode(wmode,args->output_type,args->str.s,args->clevel);
+    set->fh = hts_open(args->str.s, wmode);
     if ( set->fh == NULL ) error("[%s] Error: cannot write to \"%s\": %s\n", __func__, args->str.s, strerror(errno));
     if ( args->n_threads > 0)
         hts_set_opt(set->fh, HTS_OPT_THREAD_POOL, args->sr->p);
@@ -187,9 +194,12 @@ static void open_set(subset_t *set, args_t *args)
         if ( hts_opt_apply(set->fh, opts) ) error("Could not apply the HTS options\n");
         hts_opt_free(opts);
     }
-    bcf_hdr_t *hdr = bcf_sr_get_header(args->sr, 0);
-    if ( bcf_hdr_write(set->fh, hdr)!=0 ) error("[%s] Error: cannot write the header to %s\n", __func__, args->str.s);
-    if (args->record_cmd_line) bcf_hdr_append_version(hdr, args->argc, args->argv, "bcftools_plugin");
+    if ( !args->hdr )
+    {
+        args->hdr = bcf_sr_get_header(args->sr, 0);
+        if ( args->record_cmd_line ) bcf_hdr_append_version(args->hdr, args->argc, args->argv, "bcftools_plugin");
+    }
+    if ( bcf_hdr_write(set->fh, args->hdr)!=0 ) error("[%s] Error: cannot write the header to %s\n", __func__, args->str.s);
 }
 
 static void init_data(args_t *args)
@@ -198,11 +208,16 @@ static void init_data(args_t *args)
     if ( args->region )
     {
         args->sr->require_index = 1;
+        bcf_sr_set_opt(args->sr, BCF_SR_REGIONS_OVERLAP, args->regions_overlap);
         if ( bcf_sr_set_regions(args->sr, args->region, args->region_is_file)<0 )
             error("Failed to read the regions: %s\n", args->region);
     }
-    if ( args->target && bcf_sr_set_targets(args->sr, args->target, args->target_is_file, 0)<0 )
-        error("Failed to read the targets: %s\n", args->target);
+    if ( args->target )
+    {
+        bcf_sr_set_opt(args->sr, BCF_SR_TARGETS_OVERLAP, args->targets_overlap);
+        if ( bcf_sr_set_targets(args->sr, args->target, args->target_is_file, 0)<0 )
+            error("Failed to read the targets: %s\n", args->target);
+    }
     if ( bcf_sr_set_threads(args->sr, args->n_threads)<0 ) error("Failed to create threads\n");
     if ( !bcf_sr_add_reader(args->sr, args->fname) ) error("Error: %s\n", bcf_sr_strerror(args->sr->errnum));
 
@@ -259,7 +274,6 @@ static void destroy_data(args_t *args)
 
 static void process(args_t *args)
 {
-    bcf_hdr_t *hdr = bcf_sr_get_header(args->sr, 0);
     bcf1_t *rec = bcf_sr_get_line(args->sr, 0);
     subset_t *set;
 
@@ -272,7 +286,7 @@ static void process(args_t *args)
             open_set(set, args);
             args->rec_cnt = 0;
         }
-        if ( bcf_write(set->fh, hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
+        if ( bcf_write(set->fh, args->hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
         args->rec_cnt++;
         if (args->rec_cnt == args->nsites) {
           args->rec_cnt = 0;
@@ -282,15 +296,15 @@ static void process(args_t *args)
           args->chunk_cnt++;
         }
     } else {
-        if ( regidx_overlap(args->reg_idx, bcf_hdr_id2name(hdr, rec->rid), rec->pos, rec->pos + rec->rlen-1, args->reg_itr) ) {
+        if ( regidx_overlap(args->reg_idx, bcf_hdr_id2name(args->hdr, rec->rid), rec->pos, rec->pos, args->reg_itr) ) {
             while (regitr_overlap(args->reg_itr)) {
                 int idx = regitr_payload(args->reg_itr, int);
                 set = &args->sets[idx];
-                if ( bcf_write(set->fh, hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
+                if ( bcf_write(set->fh, args->hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
             }
         } else if (args->extra) {
             set = &args->sets[args->nsets-1];
-            if ( bcf_write(set->fh, hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
+            if ( bcf_write(set->fh, args->hdr, rec)!=0 ) error("[%s] Error: failed to write the record\n", __func__);
         }
     }
 }
@@ -301,6 +315,9 @@ int run(int argc, char **argv)
     args->argc   = argc; args->argv = argv;
     args->output_type  = FT_VCF;
     args->record_cmd_line = 1;
+    args->regions_overlap = 1;
+    args->targets_overlap = 0;
+    args->clevel = -1;
     static struct option loptions[] =
     {
         {"exclude",required_argument,NULL,'e'},
@@ -311,14 +328,16 @@ int run(int argc, char **argv)
         {"threads",required_argument,NULL,2},
         {"regions",required_argument,NULL,'r'},
         {"regions-file",required_argument,NULL,'R'},
-        {"targets", required_argument, NULL, 't'},
-        {"targets-file", required_argument, NULL, 'T'},
+        {"regions-overlap",required_argument,NULL,3},
+        {"targets", required_argument, NULL,'t'},
+        {"targets-file", required_argument, NULL,'T'},
+        {"targets-overlap",required_argument,NULL,4},
         {"nsites-per-chunk",required_argument,NULL,'n'},
         {"scatter",required_argument,NULL,'s'},
         {"scatter-file",required_argument,NULL,'S'},
         {"extra",required_argument,NULL,'x'},
         {"prefix",required_argument,NULL,'p'},
-        {"hts-opts",required_argument,NULL,3},
+        {"hts-opts",required_argument,NULL,5},
         {NULL,0,NULL,0}
     };
     int c;
@@ -336,19 +355,36 @@ int run(int argc, char **argv)
             case  1 : args->record_cmd_line = 0; break;
             case 'o': args->output_dir = optarg; break;
             case 'O':
-                      switch (optarg[0]) {
-                          case 'b': args->output_type = FT_BCF_GZ; break;
-                          case 'u': args->output_type = FT_BCF; break;
-                          case 'z': args->output_type = FT_VCF_GZ; break;
-                          case 'v': args->output_type = FT_VCF; break;
-                          default: error("The output type \"%s\" not recognised\n", optarg);
-                      }
-                      break;
+                switch (optarg[0]) {
+                    case 'b': args->output_type = FT_BCF_GZ; break;
+                    case 'u': args->output_type = FT_BCF; break;
+                    case 'z': args->output_type = FT_VCF_GZ; break;
+                    case 'v': args->output_type = FT_VCF; break;
+                    default:
+                    {
+                        args->clevel = strtol(optarg,&tmp,10);
+                        if ( *tmp || args->clevel<0 || args->clevel>9 ) error("The output type \"%s\" not recognised\n", optarg);
+                    }
+                }
+                if ( optarg[1] )
+                {
+                    args->clevel = strtol(optarg+1,&tmp,10);
+                    if ( *tmp || args->clevel<0 || args->clevel>9 ) error("Could not parse argument: --compression-level %s\n", optarg+1);
+                }
+                break;
             case  2 : args->n_threads = strtol(optarg, 0, 0); break;
             case 'r': args->region = optarg; break;
             case 'R': args->region = optarg; args->region_is_file = 1;  break;
+            case  3 :
+                args->regions_overlap = parse_overlap_option(optarg);
+                if ( args->regions_overlap < 0 ) error("Could not parse: --regions-overlap %s\n",optarg);
+                break;
             case 't': args->target = optarg; break;
             case 'T': args->target = optarg; args->target_is_file = 1; break;
+            case  4 :
+                args->targets_overlap = parse_overlap_option(optarg);
+                if ( args->targets_overlap < 0 ) error("Could not parse: --targets-overlap %s\n",optarg);
+                break;
             case 'n':
                 args->nsites = strtod(optarg, &tmp);
                 if ( tmp==optarg || *tmp ) error("Could not parse: --nsites-per-chunk %s\n", optarg);
@@ -358,7 +394,7 @@ int run(int argc, char **argv)
             case 'S': args->scatter = optarg; args->scatter_is_file = 1;  break;
             case 'x': args->extra = optarg;  break;
             case 'p': args->prefix = optarg;  break;
-            case  3 : args->hts_opts = hts_readlist(optarg, 0, &args->nhts_opts); break;
+            case  5 : args->hts_opts = hts_readlist(optarg, 0, &args->nhts_opts); break;
             case 'h':
             case '?':
             default: error("%s", usage_text()); break;
