@@ -1,19 +1,19 @@
 /* The MIT License
 
-   Copyright (c) 2019-2021 Genome Research Ltd.
+   Copyright (c) 2019-2022 Genome Research Ltd.
 
    Author: Petr Danecek <pd3@sanger.ac.uk>
-   
+
    Permission is hereby granted, free of charge, to any person obtaining a copy
    of this software and associated documentation files (the "Software"), to deal
    in the Software without restriction, including without limitation the rights
    to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
    copies of the Software, and to permit persons to whom the Software is
    furnished to do so, subject to the following conditions:
-   
+
    The above copyright notice and this permission notice shall be included in
    all copies or substantial portions of the Software.
-   
+
    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
    IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
    FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
@@ -74,7 +74,8 @@ typedef struct
 {
     convert_t *convert;
     filter_t *filter;
-    int argc, filter_logic, regions_is_file, targets_is_file, list_hdr, record_cmd_line;
+    int argc, filter_logic, regions_is_file, targets_is_file, list_hdr, record_cmd_line, clevel;
+    int regions_overlap, targets_overlap;
     kstring_t kstr;
     char *filter_str,
         *vep_tag;       // the --annotation INFO tag to process
@@ -179,7 +180,7 @@ static const char *default_column_types(void)
 }
 static const char *usage_text(void)
 {
-    return 
+    return
         "\n"
         "About: Query structured annotations such INFO/CSQ created by bcftools/csq or VEP. For more\n"
         "   more information and pointers see http://samtools.github.io/bcftools/howtos/plugin.split-vep.html\n"
@@ -195,7 +196,7 @@ static const char *usage_text(void)
         "       --columns-types -|FILE      Pass \"-\" to print the default -c types or FILE to override the presets\n"
         "   -d, --duplicate                 Output per transcript/allele consequences on a new line rather rather than\n"
         "                                     as comma-separated fields on a single line\n"
-        "   -f, --format STR                Formatting expression for non-VCF/BCF output, same as `bcftools query -f`\n"
+        "   -f, --format STR                Create non-VCF output; similar to `bcftools query -f` but drops lines w/o consequence\n"
         "   -l, --list                      Parse the VCF header and list the annotation fields\n"
         "   -p, --annot-prefix STR          Before doing anything else, prepend STR to all CSQ fields to avoid tag name conflicts\n"
         "   -s, --select TR:CSQ             Select transcripts to extract by type and/or consequence severity. (See also -S and -x.)\n"
@@ -212,11 +213,13 @@ static const char *usage_text(void)
         "   -i, --include EXPR              Include sites and samples for which the expression is true\n"
         "       --no-version                Do not append version and command line to the header\n"
         "   -o, --output FILE               Output file name [stdout]\n"
-        "   -O, --output-type b|u|z|v       b: compressed BCF, u: uncompressed BCF, z: compressed VCF or text, v: uncompressed VCF or text [v]\n"
+        "   -O, --output-type u|b|v|z[0-9]  u/b: un/compressed BCF, v/z: un/compressed VCF, 0-9: compression level [v]\n"
         "   -r, --regions REG               Restrict to comma-separated list of regions\n"
         "   -R, --regions-file FILE         Restrict to regions listed in a file\n"
+        "       --regions-overlap 0|1|2     Include if POS in the region (0), record overlaps (1), variant overlaps (2) [1]\n"
         "   -t, --targets REG               Similar to -r but streams rather than index-jumps\n"
         "   -T, --targets-file FILE         Similar to -R but streams rather than index-jumps\n"
+        "       --targets-overlap 0|1|2     Include if POS in the region (0), record overlaps (1), variant overlaps (2) [0]\n"
         "\n"
         "Examples:\n"
         "   # List available fields of the INFO/CSQ annotation\n"
@@ -246,6 +249,8 @@ static const char *usage_text(void)
         "   # the -x switch. See the online documentation referenced above for more examples.\n"
         "   bcftools +split-vep -c gnomAD_AF:Float -s :missense    file.vcf.gz\n"
         "   bcftools +split-vep -c gnomAD_AF:Float -s :missense -x file.vcf.gz\n"
+        "\n"
+        "   See also http://samtools.github.io/bcftools/howtos/plugin.split-vep.html\n"
         "\n";
 }
 
@@ -379,14 +384,52 @@ static int query_has_field(char *fmt, char *field, kstring_t *str)
     }
     return 1;
 }
+/**
+   The valid_tag array was generated with
+        perl -le '@v = (split(//,q[_.]),"a"..."z","A"..."Z","0"..."9"); @a = (0) x 256; foreach $c (@v) { $a[ord($c)] = 1; } print join(", ",@a)' | fold -w 48
+*/
+static const uint8_t valid_tag[256] =
+{
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 0,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0,
+    0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 1,
+    0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
+    1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
+    0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0
+};
+static void sanitize_field_name(char *fmt)
+{
+    while ( *fmt )
+    {
+        if ( !valid_tag[(uint8_t)*fmt] ) *fmt = '_';
+        fmt++;
+    }
+}
 char *strdup_annot_prefix(args_t *args, const char *str)
 {
-    if ( !args->annot_prefix ) return strdup(str);
+    char *out;
+    if ( !args->annot_prefix )
+    {
+        out = strdup(str);
+        sanitize_field_name(out);
+        return out;
+    }
     int str_len = strlen(str);
     int prefix_len = strlen(args->annot_prefix);
-    char *out = calloc(str_len+prefix_len+1,1);
+    out = calloc(str_len+prefix_len+1,1);
     memcpy(out,args->annot_prefix,prefix_len);
     memcpy(out+prefix_len,str,str_len);
+    sanitize_field_name(out);
     return out;
 }
 static void init_data(args_t *args)
@@ -395,9 +438,14 @@ static void init_data(args_t *args)
     if ( args->regions )
     {
         args->sr->require_index = 1;
+        bcf_sr_set_opt(args->sr,BCF_SR_REGIONS_OVERLAP,args->regions_overlap);
         if ( bcf_sr_set_regions(args->sr, args->regions, args->regions_is_file)<0 ) error("Failed to read the regions: %s\n",args->regions);
     }
-    if ( args->targets && bcf_sr_set_targets(args->sr, args->targets, args->targets_is_file, 0)<0 ) error("Failed to read the targets: %s\n",args->targets);
+    if ( args->targets )
+    {
+        bcf_sr_set_opt(args->sr,BCF_SR_TARGETS_OVERLAP,args->targets_overlap);
+        if ( bcf_sr_set_targets(args->sr, args->targets, args->targets_is_file, 0)<0 ) error("Failed to read the targets: %s\n",args->targets);
+    }
     if ( !bcf_sr_add_reader(args->sr,args->fname) ) error("Error: %s\n", bcf_sr_strerror(args->sr->errnum));
     args->hdr = bcf_sr_get_header(args->sr,0);
     args->hdr_out = bcf_hdr_dup(args->hdr);
@@ -577,7 +625,7 @@ static void init_data(args_t *args)
                 if ( keep ) ksprintf(&str,",%s",ep+1);
                 free(args->column_str);
                 args->column_str = str.s;
-                ep = str.s; 
+                ep = str.s;
                 continue;
             }
             char *tmp = strdup_annot_prefix(args, bp);
@@ -627,7 +675,7 @@ static void init_data(args_t *args)
                         ep++;
                         continue;
                     }
-                    else 
+                    else
                         error("No such column: \"%s\"\n", bp);
                 }
             }
@@ -947,7 +995,7 @@ static void process_record(args_t *args, bcf1_t *rec)
     int len = bcf_get_info_string(args->hdr,rec,args->vep_tag,&args->csq_str,&args->ncsq_str);
     if ( len<=0 )
     {
-        if ( !args->drop_sites ) 
+        if ( !args->drop_sites )
         {
             annot_reset(args->annot, args->nannot);
             filter_and_output(args,rec,1,1);
@@ -1005,7 +1053,7 @@ static void process_record(args_t *args, bcf1_t *rec)
             else
                 annot_append(ann, "."); // missing value
         }
-        
+
         if ( args->duplicate )
         {
             filter_and_output(args, rec, severity_pass, all_missing);
@@ -1027,6 +1075,9 @@ int run(int argc, char **argv)
     args->output_type  = FT_VCF;
     args->vep_tag = "CSQ";
     args->record_cmd_line = 1;
+    args->regions_overlap = 1;
+    args->targets_overlap = 0;
+    args->clevel = -1;
     static struct option loptions[] =
     {
         {"drop-sites",no_argument,0,'x'},
@@ -1046,16 +1097,19 @@ int run(int argc, char **argv)
         {"output-type",required_argument,NULL,'O'},
         {"regions",1,0,'r'},
         {"regions-file",1,0,'R'},
+        {"regions-overlap",required_argument,NULL,3},
         {"targets",1,0,'t'},
         {"targets-file",1,0,'T'},
+        {"targets-overlap",required_argument,NULL,4},
         {"no-version",no_argument,NULL,2},
         {"allow-undef-tags",no_argument,0,'u'},
         {NULL,0,NULL,0}
     };
     int c;
+    char *tmp;
     while ((c = getopt_long(argc, argv, "o:O:i:e:r:R:t:T:lS:s:c:p:a:f:dA:xu",loptions,NULL)) >= 0)
     {
-        switch (c) 
+        switch (c)
         {
             case  2 : args->record_cmd_line = 0; break;
             case  1 : args->column_types = optarg; break;
@@ -1091,9 +1145,26 @@ int run(int argc, char **argv)
                           case 'u': args->output_type = FT_BCF; break;
                           case 'z': args->output_type = FT_VCF_GZ; break;
                           case 'v': args->output_type = FT_VCF; break;
-                          default: error("The output type \"%s\" not recognised\n", optarg);
+                          default:
+                          {
+                              args->clevel = strtol(optarg,&tmp,10);
+                              if ( *tmp || args->clevel<0 || args->clevel>9 ) error("The output type \"%s\" not recognised\n", optarg);
+                          }
+                      }
+                      if ( optarg[1] )
+                      {
+                          args->clevel = strtol(optarg+1,&tmp,10);
+                          if ( *tmp || args->clevel<0 || args->clevel>9 ) error("Could not parse argument: --compression-level %s\n", optarg+1);
                       }
                       break;
+            case  3 :
+                args->regions_overlap = parse_overlap_option(optarg);
+                if ( args->regions_overlap < 0 ) error("Could not parse: --regions-overlap %s\n",optarg);
+                break;
+            case  4 :
+                args->targets_overlap = parse_overlap_option(optarg);
+                if ( args->targets_overlap < 0 ) error("Could not parse: --targets-overlap %s\n",optarg);
+                break;
             case 'h':
             case '?':
             default: error("%s", usage_text()); break;
@@ -1129,7 +1200,9 @@ int run(int argc, char **argv)
             args->fh_bgzf = bgzf_open(args->output_fname, args->output_type&FT_GZ ? "wg" : "wu");
         else
         {
-            args->fh_vcf = hts_open(args->output_fname, hts_bcf_wmode2(args->output_type,args->output_fname));
+            char wmode[8];
+            set_wmode(wmode,args->output_type,args->output_fname,args->clevel);
+            args->fh_vcf = hts_open(args->output_fname ? args->output_fname : "-", wmode);
             if ( args->record_cmd_line ) bcf_hdr_append_version(args->hdr_out, args->argc, args->argv, "bcftools_split-vep");
             if ( bcf_hdr_write(args->fh_vcf, args->hdr_out)!=0 ) error("Failed to write the header to %s\n", args->output_fname);
         }
