@@ -61,7 +61,8 @@ typedef struct
     int filter_logic;   // one of FLT_INCLUDE/FLT_EXCLUDE (-i or -e)
     uint8_t *info_tags, *fmt_tags;
     int ninfo_tags, minfo_tags, nfmt_tags, mfmt_tags, keep_info, keep_fmt;
-    int argc, region_is_file, target_is_file, output_type;
+    int argc, region_is_file, target_is_file, output_type, clevel;
+    int regions_overlap, targets_overlap;
     char **argv, *region, *target, *fname, *output_dir, *keep_tags, *samples_fname, *groups_fname;
     void *unique_fnames;
     bcf_hdr_t *hdr_in, *hdr_out;
@@ -86,8 +87,8 @@ static const char *usage_text(void)
         "       with \"_\", and a unique numeric suffix added in case of name clashes.\n"
         "Usage: bcftools +split [Options]\n"
         "Plugin options:\n"
-        "   -e, --exclude EXPR              exclude sites for which the expression is true (applied on the outputs)\n"
-        "   -G, --groups-file FILE          similar to -S, but the samples are split by group:\n"
+        "   -e, --exclude EXPR              Exclude sites for which the expression is true (applied on the outputs)\n"
+        "   -G, --groups-file FILE          Similar to -S, but the samples are split by group:\n"
         "                                       \n"
         "                                       # Create two output files (third column) with the second sample appearing\n"
         "                                       # in both. The second column is for optional renaming of the samples, use\n"
@@ -96,13 +97,14 @@ static const char *usage_text(void)
         "                                       sample2   -          file1,file2\n"
         "                                       sample3   new-name3  file2\n"
         "                                       \n"
-        "   -i, --include EXPR              include only sites for which the expression is true (applied on the outputs)\n"
-        "   -k, --keep-tags LIST            list of tags to keep. By default all tags are preserved\n"
-        "   -o, --output DIR                write output to the directory DIR\n"
-        "   -O, --output-type b|u|z|v       b: compressed BCF, u: uncompressed BCF, z: compressed VCF, v: uncompressed VCF [v]\n"
-        "   -r, --regions REGION            restrict to comma-separated list of regions\n"
-        "   -R, --regions-file FILE         restrict to regions listed in a file\n"
-        "   -S, --samples-file FILE         list of samples to keep with up to three columns, one line per output file:\n"
+        "   -i, --include EXPR              Include only sites for which the expression is true (applied on the outputs)\n"
+        "   -k, --keep-tags LIST            List of tags to keep. By default all tags are preserved\n"
+        "   -o, --output DIR                Write output to the directory DIR\n"
+        "   -O, --output-type u|b|v|z[0-9]  u/b: un/compressed BCF, v/z: un/compressed VCF, 0-9: compression level [v]\n"
+        "   -r, --regions REGION            Restrict to comma-separated list of regions\n"
+        "   -R, --regions-file FILE         Restrict to regions listed in a file\n"
+        "       --regions-overlap 0|1|2     Include if POS in the region (0), record overlaps (1), variant overlaps (2) [1]\n"
+        "   -S, --samples-file FILE         List of samples to keep with up to three columns, one line per output file:\n"
         "                                       \n"
         "                                       # Create two output files, the first sample is the basename\n"
         "                                       # of the new file\n"
@@ -118,9 +120,10 @@ static const char *usage_text(void)
         "                                       sample1           new-name1   file1\n"
         "                                       sample2,sample3   -           file2\n"
         "                                       \n"
-        "   -t, --targets REGION            similar to -r but streams rather than index-jumps\n"
-        "   -T, --targets-file FILE         similar to -R but streams rather than index-jumps\n"
-        "       --hts-opts LIST             low-level options to pass to HTSlib, e.g. block_size=32768\n"
+        "   -t, --targets REGION            Similar to -r but streams rather than index-jumps\n"
+        "       --targets-overlap 0|1|2     Include if POS in the region (0), record overlaps (1), variant overlaps (2) [0]\n"
+        "   -T, --targets-file FILE         Similar to -R but streams rather than index-jumps\n"
+        "       --hts-opts LIST             Low-level options to pass to HTSlib, e.g. block_size=32768\n"
         "\n"
         "Examples:\n"
         "   # Split a VCF file\n"
@@ -361,9 +364,14 @@ static void init_data(args_t *args)
     if ( args->region )
     {
         args->sr->require_index = 1;
+        bcf_sr_set_opt(args->sr,BCF_SR_REGIONS_OVERLAP,args->regions_overlap);
         if ( bcf_sr_set_regions(args->sr, args->region, args->region_is_file)<0 ) error("Failed to read the regions: %s\n",args->region);
     }
-    if ( args->target && bcf_sr_set_targets(args->sr, args->target, args->target_is_file, 0)<0 ) error("Failed to read the targets: %s\n",args->target);
+    if ( args->target )
+    {
+        bcf_sr_set_opt(args->sr,BCF_SR_TARGETS_OVERLAP,args->targets_overlap);
+        if ( bcf_sr_set_targets(args->sr, args->target, args->target_is_file, 0)<0 ) error("Failed to read the targets: %s\n",args->target);
+    }
     if ( !bcf_sr_add_reader(args->sr,args->fname) ) error("Error: %s\n", bcf_sr_strerror(args->sr->errnum));
     args->hdr_in  = bcf_sr_get_header(args->sr,0);
 
@@ -451,7 +459,7 @@ static void init_data(args_t *args)
         if ( str.s[str.l-1] != '/' ) kputc('/', &str);
         kputs(set->fname, &str);
         char *suffix = NULL;
-        if ( args->output_type & FT_BCF ) suffix = "bcf";
+        if ( args->output_type & FT_BCF ) suffix = ".bcf";
         else if ( args->output_type & FT_GZ ) suffix = ".vcf.gz";
         else suffix = ".vcf";
         int len = strlen(set->fname);
@@ -460,7 +468,9 @@ static void init_data(args_t *args)
         if ( len >= 7 && !strcasecmp(".vcf.gz",set->fname+len-7) ) suffix = NULL;
         if ( len >= 8 && !strcasecmp(".vcf.bgz",set->fname+len-8) ) suffix = NULL;
         if ( suffix ) kputs(suffix, &str);
-        set->fh = hts_open(str.s, hts_bcf_wmode2(args->output_type,str.s));
+        char wmode[8];
+        set_wmode(wmode,args->output_type,str.s,args->clevel);
+        set->fh = hts_open(str.s, wmode);
         if ( set->fh == NULL ) error("[%s] Error: cannot write to \"%s\": %s\n", __func__, str.s, strerror(errno));
         if ( args->hts_opts )
         {
@@ -612,6 +622,9 @@ int run(int argc, char **argv)
     args_t *args = (args_t*) calloc(1,sizeof(args_t));
     args->argc   = argc; args->argv = argv;
     args->output_type  = FT_VCF;
+    args->regions_overlap = 1;
+    args->targets_overlap = 0;
+    args->clevel = -1;
     static struct option loptions[] =
     {
         {"hts-opts",required_argument,NULL,1},
@@ -620,6 +633,10 @@ int run(int argc, char **argv)
         {"include",required_argument,NULL,'i'},
         {"regions",required_argument,NULL,'r'},
         {"regions-file",required_argument,NULL,'R'},
+        {"regions-overlap",required_argument,NULL,2},
+        {"targets",required_argument,NULL,'t'},
+        {"targets-file",required_argument,NULL,'T'},
+        {"targets-overlap",required_argument,NULL,3},
         {"samples-file",required_argument,NULL,'S'},
         {"groups-file",required_argument,NULL,'G'},
         {"output",required_argument,NULL,'o'},
@@ -627,6 +644,7 @@ int run(int argc, char **argv)
         {NULL,0,NULL,0}
     };
     int c;
+    char *tmp;
     while ((c = getopt_long(argc, argv, "vr:R:t:T:o:O:i:e:k:S:G:",loptions,NULL)) >= 0)
     {
         switch (c) 
@@ -652,9 +670,26 @@ int run(int argc, char **argv)
                           case 'u': args->output_type = FT_BCF; break;
                           case 'z': args->output_type = FT_VCF_GZ; break;
                           case 'v': args->output_type = FT_VCF; break;
-                          default: error("The output type \"%s\" not recognised\n", optarg);
+                          default:
+                          {
+                              args->clevel = strtol(optarg,&tmp,10);
+                              if ( *tmp || args->clevel<0 || args->clevel>9 ) error("The output type \"%s\" not recognised\n", optarg);
+                          }
+                      }
+                      if ( optarg[1] )
+                      {
+                          args->clevel = strtol(optarg+1,&tmp,10);
+                          if ( *tmp || args->clevel<0 || args->clevel>9 ) error("Could not parse argument: --compression-level %s\n", optarg+1);
                       }
                       break;
+            case  2 :
+                args->regions_overlap = parse_overlap_option(optarg);
+                if ( args->regions_overlap < 0 ) error("Could not parse: --regions-overlap %s\n",optarg);
+                break;
+            case  3 :
+                args->targets_overlap = parse_overlap_option(optarg);
+                if ( args->targets_overlap < 0 ) error("Could not parse: --targets-overlap %s\n",optarg);
+                break;
             case 'h':
             case '?':
             default: error("%s", usage_text()); break;
